# -*- coding: utf-8 -*-
#==========================================================#
#Last Updated - March 2018
#@author: z5025317 Valentin Heimhuber
#code for creating future climate variability deviation plots for NARCLIM variables. 
#Inputs: Uses CSV files that containthe deltas of all 12 NARCLIM models for 1 grid cell at the site of interest, generated with P1_NARCLIM_plots_Windows.py
#This code is used only for the NARCLIM variables - a separate code is used for ocean variables etc that are not part of the NARCLIM ensemble

#==========================================================#
#Load packages
#==========================================================#
import numpy as np
import os
import pandas as pd
import glob
import matplotlib
import matplotlib.pyplot as plt
from datetime import datetime
from datetime import timedelta
from matplotlib.backends.backend_pdf import PdfPages
from ggplot import *
import csv
matplotlib.style.use('ggplot')
# Load my own functions
os.chdir('C:/Users/z5025317/OneDrive - UNSW/WRL_Postdoc_Manual_Backup/WRL_Postdoc/Projects/Paper#1/Analysis/Code')
import climdata_fcts as fct
import silo as sil
#==========================================================#

#==========================================================#
# Set working direcotry (where postprocessed NARClIM data is located)
os.chdir('C:/Users/z5025317/OneDrive - UNSW/WRL_Postdoc_Manual_Backup/WRL_Postdoc/Projects/Paper#1/')
#==========================================================#
     
#loop through case study estuaries    
Estuaries = ['HUNTER', 'RICHMOND', 'NADGEE', 'SHOALHAVEN', 'GEORGES','CATHIE']
Estuaries = ['HUNTER']
for Est in Estuaries:
    
    #==========================================================#
    #set input parameters
    #==========================================================#
    Case_Study_Name = 'CASESTUDY2' 
    Casestudy2_csv_path =  "Data/NARCLIM_Site_CSVs/CASESTUDY2/CASESTDUY2_NARCLIM_Point_Sites.csv"
    #Estuary = 'HUNTER' # 'Belongil'
    Estuary = Est # 'Belongil'
    print Estuary
    Base_period_start = '1970-01-01'        #Start of interval for base period of climate variability
    Base_period_DELTA_start = '1990-01-01'   #Start of interval used for calculating mean and SD for base period (Delta base period)
    Base_period_end = '2009-01-01'          #use last day that's not included in period as < is used for subsetting
    Clim_var_type  = 'tasmax'            #Name of climate variable in NARCLIM models  '*' will create pdf for all variables in folder
      
    PD_Datasource = 'SILO'                  #Source for present day climate data (historic time series) can be either: 'Station' or 'SILO'
    SILO_Clim_var = ['max_temp']         #select the SILO clim variable to be used for base period. - see silo.py for detailed descriptions
    Location = 'Estuary'                 # pick locaiton for extracting the SILO data can be: Estuary, Catchment, or Ocean
    
    presentdaybar = False               #include a bar for present day variability in the plots?
    present_day_plot = 'no'             #save a time series of present day        
    Version = "V1"
    Stats = 'days_h_35'                  #'maxdaily' #maximum takes the annual max Precipitation instead of the sum
    ALPHA_figs = 1                      #Set alpha of figure background (0 makes everything around the plot panel transparent)
    #==========================================================#
    
    
    
    #==========================================================#
    #set directory path for output files
    output_directory = 'Output/' + Case_Study_Name + '/' + Estuary + '/' + '/Clim_Deviation_Plots/'
    #output_directory = 'J:/Project wrl2016032/NARCLIM_Raw_Data/Extracted'
    if not os.path.exists(output_directory):
        os.makedirs(output_directory)
        print('-------------------------------------------')
        print("output directory folder didn't exist and was generated")
        print('-------------------------------------------')
        print('-------------------')
    #==========================================================#
    
    
    #==========================================================#
    #read CSV file  
    #==========================================================#
    Clim_Var_CSVs = glob.glob('./Output/' + Case_Study_Name + '/'  + Estuary + '/' + Estuary + '_' + Clim_var_type + '_' + Stats + '*')
    clim_var_csv_path = Clim_Var_CSVs[0]
    Filename = os.path.basename(os.path.normpath(clim_var_csv_path))
    Clim_var_type = Filename.split('_', 2)[1]
    print(Clim_var_type)
    Ensemble_Delta_full_df = pd.read_csv(clim_var_csv_path, index_col=0, parse_dates = True)
    
    if Clim_var_type == 'rsdsmean':
        Clim_Var_CSVs = glob.glob('./Output/' + Case_Study_Name + '/'  + Estuary + '/' + Estuary + '_rsdsmean_' + Stats + '*')
        clim_var_csv_path = Clim_Var_CSVs[0]
        Filename = os.path.basename(os.path.normpath(clim_var_csv_path))
        Clim_var_type = Filename.split('_', 2)[1]
        print(Clim_var_type)
        Ensemble_Delta_full_df1 = pd.read_csv(clim_var_csv_path, index_col=0, parse_dates = True)
        Clim_Var_CSVs = glob.glob('./Output/' + Case_Study_Name + '/'  + Estuary + '/' + Estuary + '_rldsmean_' + Stats + '*')
        clim_var_csv_path = Clim_Var_CSVs[0]
        Filename = os.path.basename(os.path.normpath(clim_var_csv_path))
        Clim_var_type = Filename.split('_', 2)[1]
        print(Clim_var_type)
        Ensemble_Delta_full_df2 = pd.read_csv(clim_var_csv_path, index_col=0, parse_dates = True)
        Ensemble_Delta_full_df = Ensemble_Delta_full_df2 + Ensemble_Delta_full_df1
    #Ensemble_Delta_full_df = pd.to_numeric(Ensemble_Delta_full_df)
    #==========================================================# 
    
    
    #==========================================================#
    #load present day climate variable time series
    #==========================================================#
    if PD_Datasource == 'Station':
        Present_day_df,minplotDelta, maxplotDelta = fct.import_present_day_climdata_csv(Estuary, Clim_var_type)
    
    if PD_Datasource == 'SILO':
        #read the CSV to extract the lat long and case stuy sites
        with open(Casestudy2_csv_path, mode='r') as infile:
            reader = csv.reader(infile)
            next(reader, None) 
            with open('coors_new.csv', mode='w') as outfile:
                writer = csv.writer(outfile)
                if Location == 'Estuary':
                    mydict = dict((rows[0],[rows[1],rows[2]]) for rows in reader)
                if Location == 'Ocean':
                    mydict = dict((rows[0],[rows[3],rows[4]]) for rows in reader)
                if Location == 'Catchment':
                    mydict = dict((rows[0],[rows[5],rows[6]]) for rows in reader)
                
        if Clim_var_type == 'tasmean':
            silo_df = sil.pointdata(["max_temp", "min_temp"], 'Okl9EDxgS2uzjLWtVNIBM5YqwvVcCxOmpd3nCzJh', Base_period_start.replace("-", ""), Base_period_end.replace("-", ""), 
                                        None, mydict[Estuary][0], mydict[Estuary][1], False, None)
            #take mean of daily min and max temp
            Present_day_df =     silo_df.iloc[:,[2,4]].mean(axis=1)
        else:
            silo_df = sil.pointdata(SILO_Clim_var, 'Okl9EDxgS2uzjLWtVNIBM5YqwvVcCxOmpd3nCzJh', Base_period_start.replace("-", ""), Base_period_end.replace("-", ""), 
                                        None, mydict[Estuary][0], mydict[Estuary][1], False, None)
            Present_day_df = silo_df.iloc[:,[2]]
    
        #set the x and y limit deltas -  for plotting only
        if Clim_var_type in ['evspsblmean', 'potevpmean']: #ET time series that we have is not in the same format as the other variables, hence the different treatment
            [minplotDelta, maxplotDelta]=[50,50]
        #for tasmean, observed min and max T need to be converted into mean T
        elif Clim_var_type == 'tasmean':
            [minplotDelta, maxplotDelta]=[0.2,1]
        elif Clim_var_type == 'tasmax':
            [minplotDelta, maxplotDelta]=[1,2]  
        elif Clim_var_type == 'wssmean' or  Clim_var_type == 'wss1Hmaxtstep':
            [minplotDelta, maxplotDelta]=[1, 1.5]  
        elif Clim_var_type == 'pracc':   
            [minplotDelta, maxplotDelta]=[50,100]
        elif Clim_var_type in ['rsdsmean', 'rldsmean']:   
            [minplotDelta, maxplotDelta]=[100,100]
            #[minplotDelta, maxplotDelta]=[1,1]
    #==========================================================#
    
     #substract a constant from all values to convert from kelvin to celcius (temp)
    if Clim_var_type == 'sstmean':
        Present_day_df = Present_day_df.iloc[:,0:(len(Present_day_df)-1)]-273.15
        Present_day_df.index = Present_day_df.index.to_period('D')
    #==========================================================#
    #create seasonal sums etc. 
    #==========================================================#
    if Stats == 'maxdaily':
        Present_day_df_annual = Present_day_df.resample('A').max()
        Present_day_df_annual = Present_day_df_annual.replace(0, np.nan)
        Fdf_Seas_means = Present_day_df.resample('Q-NOV').max() #seasonal means
        Fdf_Seas_means = Fdf_Seas_means.replace(0, np.nan)
    if(Stats[:4] =='days'):
        Threshold = int(Stats[-2:])
        agg = ('>'+ str(Threshold) + '_count', lambda x: x.gt(Threshold).sum()),
        Present_day_df_annual = Present_day_df.resample('A').agg(agg)                
        Fdf_Seas_means = Present_day_df.resample('Q-NOV').agg(agg)  #seasonal means
    else:
        if Clim_var_type in ['pracc' ,'evspsblmean' ,'potevpmean' ,
                         'pr1Hmaxtstep' ,'wss1Hmaxtstep',
                         'rsdsmean', 'rldsmean']:
            Present_day_df_annual = Present_day_df.resample('A').sum()
            Present_day_df_annual = Present_day_df_annual.replace(0, np.nan)
            Fdf_Seas_means = Present_day_df.resample('Q-NOV').sum() #seasonal means
            Fdf_Seas_means = Fdf_Seas_means.replace(0, np.nan)
        elif Clim_var_type in ['tasmean' ,'tasmax' ,'sstmean']:    
            Present_day_df_annual = Present_day_df.resample('A').mean()
            Present_day_df_monthly = Present_day_df.resample('M').mean()
            Present_day_df_weekly = Present_day_df.resample('W').mean()
            Fdf_Seas_means = Present_day_df.resample('Q-NOV').mean() #seasonal means
    #==========================================================#
      
    
    #==========================================================#   
    #Loop through annual and seasons and create a deviation plot for each. 
    #==========================================================#
    times = ['annual', 'DJF', 'MAM', 'JJA','SON']
    fig = plt.figure(figsize=(14,8))
    delta_all_df = pd.DataFrame()
    i=1
    for temp in times:
        #temp = 'annual'
        #subset the ensemble dataframe for the period used:
        if temp == 'annual':
            Ensemble_Delta_df = Ensemble_Delta_full_df.iloc[:,range(0,2)]
            Present_Day_ref_df = Present_day_df_annual
        else:
            Ensemble_Delta_df = Ensemble_Delta_full_df.filter(regex= temp)
            Ensemble_Delta_df.columns = ['near', 'far']
            if temp == 'DJF':
                Mean_df = Fdf_Seas_means[Fdf_Seas_means.index.quarter==1]
            Column_names = ['DJF_near', 'DJF_far']
            if temp == 'MAM':
                Mean_df = Fdf_Seas_means[Fdf_Seas_means.index.quarter==2]
            Column_names = ['MAM_near', 'MAM_far']
            if temp == 'JJA':
                Mean_df = Fdf_Seas_means[Fdf_Seas_means.index.quarter==3]
            Column_names = ['JJA_near', 'JJA_far']
            if temp == 'SON':
                Mean_df = Fdf_Seas_means[Fdf_Seas_means.index.quarter==4]
            Present_Day_ref_df = Mean_df
        #Subset to present day variability period
        Present_Day_ref_df = pd.DataFrame(Present_Day_ref_df.loc[(Present_Day_ref_df.index >= Base_period_start) & (Present_Day_ref_df.index <= Base_period_end)])
        #Subset to present day variability delta base period (this is the statistical baseline used for present day conditions)
        Present_Day_Delta_ref_df = pd.DataFrame(Present_Day_ref_df.loc[(Present_Day_ref_df.index >= Base_period_DELTA_start) & (Present_Day_ref_df.index <= Base_period_end)])
        Present_Day_Mean = np.percentile(Present_Day_Delta_ref_df, 50)
        Present_Day_SD = np.std(Present_Day_Delta_ref_df)
        #create data frame for floating stacked barplots 
        index=['-2std', '-1std', 'Med', '1std', '2std']
        columns = ['present','near future', 'far future']
        Plot_in_df = pd.DataFrame(index=index, columns =columns)
        #  
        Plot_in_df['present'] = [float(Present_Day_Mean-2*Present_Day_SD),float(Present_Day_Mean-Present_Day_SD), float(Present_Day_Mean),
               float(Present_Day_Mean+Present_Day_SD), float(Present_Day_Mean+2*Present_Day_SD)]
        Plot_in_df['near future'] = [float(Present_Day_Mean + Ensemble_Delta_df.near[-3:][0]),np.NaN, float(Present_Day_Mean + Ensemble_Delta_df.near[-3:][1]), 
               np.NaN, float(Present_Day_Mean + Ensemble_Delta_df.near[-3:][2])]
        Plot_in_df['far future'] = [float(Present_Day_Mean + Ensemble_Delta_df.far[-3:][0]),np.NaN, float(Present_Day_Mean + Ensemble_Delta_df.far[-3:][1]), 
               np.NaN, float(Present_Day_Mean + Ensemble_Delta_df.far[-3:][2])]
        #Create a second data frame that has the values in a way that they can be stacked up in bars with the correct absolute values
        Plot_in_df2 = pd.DataFrame(index=index, columns =columns )
        #
        if presentdaybar == False:
            index=['-2std', '-1std', 'Med', '1std', '2std']
            columns = ['near future', 'far future']
            Plot_in_df2 = pd.DataFrame(index=index, columns =columns )
            #Plot_in_df2['present'] = [float(0),float(0), float(0),
            #              float(0), float(0)]  
        else:
            Plot_in_df2['present'] = [float(Present_Day_Mean-2*Present_Day_SD),float(Present_Day_SD), float(Present_Day_SD),
                    float(Present_Day_SD), float(Present_Day_SD)]
        Plot_in_df2['near future'] = [float(Present_Day_Mean + Ensemble_Delta_df.near[-3:][0]),np.NaN, float(Ensemble_Delta_df.near[-3:][1]-Ensemble_Delta_df.near[-3:][0]), 
              np.NaN, float(Ensemble_Delta_df.near[-3:][2]-Ensemble_Delta_df.near[-3:][1])]
        Plot_in_df2['far future'] = [float(Present_Day_Mean + Ensemble_Delta_df.far[-3:][0]),np.NaN, float(Ensemble_Delta_df.far[-3:][1]-Ensemble_Delta_df.far[-3:][0]), 
               np.NaN, float(Ensemble_Delta_df.far[-3:][2]-Ensemble_Delta_df.far[-3:][1])]
        #transpose the data frame
        Plot_in_df_tp = Plot_in_df2.transpose()
        #do the individual plots
        if temp == 'annual':
            xmin = int(min(Plot_in_df.min(axis=1))-minplotDelta)
            xmax = int(max(Plot_in_df.max(axis=1))+maxplotDelta)
        else:
            xmin = int(min(Plot_in_df.min(axis=1))-minplotDelta)
            xmax = int(max(Plot_in_df.max(axis=1))+maxplotDelta)
        #define colour scheme
        #likert_colors = ['none', 'firebrick','firebrick','lightcoral','lightcoral']
        likert_colors = ['none', 'darkblue', 'darkblue','cornflowerblue','cornflowerblue']
        uni_colors = ['none', 'cornflowerblue', 'cornflowerblue','cornflowerblue','cornflowerblue']
        #plot the stacked barplot
        if temp == 'annual':
            ax=plt.subplot(2,4,3) 
        else:
            ax=plt.subplot(2,4,i)  
        Plot_in_df_tp.plot.bar(stacked=True, color=uni_colors, edgecolor='none', legend=False, ax=ax, width=0.5)  
        df = pd.DataFrame(Plot_in_df.iloc[2,[1,2]])
        index2=['Med']
        columns2 = ['near future', 'far future']
        Plot_in_df3 = pd.DataFrame(index=index2, columns =columns2)
        Plot_in_df3['near future'] = df.iloc[1,0]
        Plot_in_df3['far future'] = df.iloc[0,0]
        Plot_in_df3 = Plot_in_df3.transpose()
        plt.plot(Plot_in_df3['Med'], linestyle="", markersize=52, 
             marker="_", color='darkblue', label="Median")
        z = plt.axhline(float(Present_Day_Mean-2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean+2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean-Present_Day_SD), linestyle='--', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean+Present_Day_SD), linestyle='--', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean), linestyle='--', color='red', alpha=.5)
        z.set_zorder(-1)
        plt.ylim(xmin, xmax)
        plt.title(Clim_var_type + ' ' + temp )
        ax.grid(False)
        for tick in ax.get_xticklabels():
            tick.set_rotation(0)
        fig.tight_layout()
        fig.patch.set_alpha(ALPHA_figs)
                 
        if temp == 'annual':
            ax=plt.subplot(2,2,1) 
            Present_Day_ref_df.plot(legend=False, ax=ax)
            z = plt.axhline(float(Present_Day_Mean-2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
            z.set_zorder(-1)
            z = plt.axhline(float(Present_Day_Mean+2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
            z.set_zorder(-1)
            z = plt.axhline(float(Present_Day_Mean-Present_Day_SD), linestyle='--', color='black', alpha=.5)
            z.set_zorder(-1)
            z = plt.axhline(float(Present_Day_Mean+Present_Day_SD), linestyle='--', color='black', alpha=.5)
            z.set_zorder(-1)
            z = plt.axhline(float(Present_Day_Mean), linestyle='--', color='red', alpha=.5)
            z.set_zorder(-1)
            #fig.patch.set_facecolor('deepskyblue')
            fig.tight_layout()
            fig.patch.set_alpha(ALPHA_figs)
            plt.title(Clim_var_type + ' ' + Stats + ' ' + temp +' present day')
            plt.ylim(xmin, xmax)
            ax.grid(False)
            #if temp == 'MAM':
            i=i+4
        else:
            i=i+1
       
    #plt.show()   
    if presentdaybar == False:
        out_file_name = Estuary  + '_' + Clim_var_type + '_' + Stats + '_'  + PD_Datasource +  '_'  + SILO_Clim_var[0] +  Version +  '_'  + '_NPDB.png'
    else:
        out_file_name = Estuary  + '_' + Clim_var_type + '_' + Stats + '_'  + PD_Datasource +  '_'  + SILO_Clim_var[0] +  Version +  '_'  + '.png'
    out_path = output_directory + '/' + out_file_name
    fig.savefig(out_path)
        
    if present_day_plot == 'yes':  
        #print present day climate data
        fig = plt.figure(figsize=(5,4))
        ax = fig.add_subplot(1, 1, 1)
        if temp == 'annual':
            xmin = int(min(Plot_in_df.min(axis=1))-minplotDelta)
            xmax = int(max(Plot_in_df.max(axis=1))+maxplotDelta)
        else:
            xmin = int(min(Plot_in_df.min(axis=1))-minplotDelta)
            xmax = int(max(Plot_in_df.max(axis=1))+maxplotDelta)
        
        Present_Day_ref_df.plot(legend=False, ax=ax)
        z = plt.axhline(float(Present_Day_Mean-2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean+2*Present_Day_SD), linestyle='-', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean-Present_Day_SD), linestyle='--', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean+Present_Day_SD), linestyle='--', color='black', alpha=.5)
        z.set_zorder(-1)
        z = plt.axhline(float(Present_Day_Mean), linestyle='--', color='red', alpha=.5)
        z.set_zorder(-1)
        #fig.patch.set_facecolor('deepskyblue')
        fig.patch.set_alpha(0)
        plt.ylim(13, xmax)
        #export plot to png
        out_file_name = Estuary  + '_' + Clim_var_type + '_' + Stats + '_'  + Base_period_start +  '_'  + Base_period_end +  Version + 'Present_Day_Period.png'
        out_path = output_directory + '/' + out_file_name
        fig.savefig(out_path)
        # use transparent=True if you want the whole figure with a transparent background
        
        
    
    
    
    
    
    
    
    